@@ -39,8 +39,8 @@ from typing import (
     TypeVar,
     cast,
 )
+from warnings import warn
 from weakref import ref as weak_ref
-from weakref import ref as weakref
 
 from .dbus_common import (
     DbusMethodCommon,
@@ -48,10 +48,12 @@ from .dbus_common import (
     DbusSomethingSync,
     _method_name_converter,
     get_default_bus,
+    is_property_flags_correct,
 )
 from .dbus_exceptions import DbusFailedError
 from .sd_bus_internals import (
     DbusNoReplyFlag,
+    DbusPropertyEmitsChangeFlag,
     SdBus,
     SdBusInterface,
     SdBusMessage,
@@ -79,7 +81,7 @@ class DbusMethodAsyncBinded(DbusBindedAsync):
                  dbus_method: DbusMethodAsync,
                  interface: DbusInterfaceBaseAsync):
         self.dbus_method = dbus_method
-        self.interface_ref = weakref(interface)
+        self.interface_ref = weak_ref(interface)
 
         self.__doc__ = dbus_method.__doc__
 
@@ -169,8 +171,15 @@ class DbusMethodAsyncBinded(DbusBindedAsync):
         reply_message = request_message.create_reply()
 
         if isinstance(reply_data, tuple):
-            reply_message.append_data(
-                self.dbus_method.result_signature, *reply_data)
+            try:
+                reply_message.append_data(
+                    self.dbus_method.result_signature, *reply_data)
+            except TypeError:
+                # In case of single struct result type
+                # We can't figure out if return is multiple values
+                # or a tuple
+                reply_message.append_data(
+                    self.dbus_method.result_signature, reply_data)
         elif reply_data is not None:
             reply_message.append_data(
                 self.dbus_method.result_signature, reply_data)
@@ -229,6 +238,8 @@ class DbusProperty(DbusSomethingAsync, Generic[T]):
             flags: int,
 
     ) -> None:
+        is_property_flags_correct(flags)  # Check that number of passed flags
+
         super().__init__()
         self.property_name = property_name
         self.property_signature = property_signature
@@ -236,6 +247,9 @@ class DbusProperty(DbusSomethingAsync, Generic[T]):
         self.property_setter = property_setter
         self.flags = flags
 
+        self.properties_changed_signal: \
+            Optional[DbusSignalBinded[DBUS_PROPERTIES_CHANGED_TYPING]] = None
+
         self.__doc__ = property_getter.__doc__
 
 
@@ -262,7 +276,7 @@ class DbusPropertyAsyncBinded(DbusBindedAsync):
                  dbus_property: DbusPropertyAsync[T],
                  interface: DbusInterfaceBaseAsync):
         self.dbus_property = dbus_property
-        self.interface_ref = weakref(interface)
+        self.interface_ref = weak_ref(interface)
 
         self.__doc__ = dbus_property.__doc__
 
@@ -311,6 +325,21 @@ class DbusPropertyAsyncBinded(DbusBindedAsync):
 
         self.dbus_property.property_setter(interface, data_to_set_to)
 
+        if self.dbus_property.properties_changed_signal is not None:
+            assert self.dbus_property.interface_name is not None
+            self.dbus_property.properties_changed_signal.emit(
+                (
+                    self.dbus_property.interface_name,
+                    {
+                        self.dbus_property.property_name: (
+                            self.dbus_property.property_signature,
+                            data_to_set_to,
+                        ),
+                    },
+                    []
+                )
+            )
+
     async def set_async(self, complete_object: T) -> None:
         interface = self.interface_ref()
         assert interface is not None
@@ -416,7 +445,7 @@ class DbusSignalBinded(Generic[T], DbusBindedAsync):
                  dbus_signal: DbusSignal[T],
                  interface: DbusInterfaceBaseAsync):
         self.dbus_signal = dbus_signal
-        self.interface_ref = weakref(interface)
+        self.interface_ref = weak_ref(interface)
 
         self.__doc__ = dbus_signal.__doc__
 
@@ -655,7 +684,6 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
     _dbus_interface_name: Optional[str]
     _dbus_serving_enabled: bool
 
-    # TODO: make all attributes start with _
     def __init__(self) -> None:
         self._activated_interfaces: List[SdBusInterface] = []
         self._is_binded: bool = False
@@ -671,7 +699,6 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
                             bus: Optional[SdBus] = None,
                             ) -> None:
 
-        from warnings import warn
         warn("start_serving is deprecated in favor of export_to_dbus",
              DeprecationWarning)
         self.export_to_dbus(object_path, bus)
@@ -791,11 +818,16 @@ class DbusInterfaceBaseAsync(metaclass=DbusInterfaceMetaAsync):
         object_path: str,
         bus: Optional[SdBus] = None,
     ) -> T_input:
-        return cls.new_proxy(
-            service_name,
-            object_path,
-            bus,
+        warn(
+            ("new_connect is deprecated in favor of equivalent new_proxy."
+             "Will be removed in version 1.0.0"),
+            DeprecationWarning,
         )
+        new_object = cls.__new__(cls)
+        assert isinstance(new_object, DbusInterfaceBaseAsync)
+        new_object._proxify(service_name, object_path, bus)
+        assert isinstance(new_object, cls)
+        return new_object
 
     @classmethod
     def new_proxy(
@@ -838,15 +870,30 @@ class DbusIntrospectableAsync(
         raise NotImplementedError
 
 
+DBUS_PROPERTIES_CHANGED_TYPING = Tuple[str,
+                                       Dict[str, Tuple[str, Any]],
+                                       List[str]]
+
+
 class DbusPropertiesInterfaceAsync(
     DbusInterfaceBaseAsync,
     interface_name='org.freedesktop.DBus.Properties',
     serving_enabled=False,
 ):
+    def __init__(self) -> None:
+        super().__init__()
+
+        properties_changed_signal = self.properties_changed
+        for key, value in getmembers(self):
+            if isinstance(value, DbusPropertyAsyncBinded):
+                if not value.dbus_property.flags & DbusPropertyEmitsChangeFlag:
+                    continue
+
+                value.dbus_property.properties_changed_signal \
+                    = properties_changed_signal
+
     @dbus_signal_async('sa{sv}as')
-    def properties_changed(self) -> Tuple[str,
-                                          Dict[str, Tuple[str, Any]],
-                                          List[str]]:
+    def properties_changed(self) -> DBUS_PROPERTIES_CHANGED_TYPING:
         ...
 
 
